#######################################################################################
# File: 			totd.tcl
# Purpose:			Tip of the Day for Ticol autoexec.tcl (include file)
# Language:			Ticol Tcl v1.26+
#
# Versions:
# 1.00 22/Oct/2019	First version
# 1.01 17/Feb/2021	Updated
# 1.02 02/Mar/2021	Updated some more
# 1.03 09/Oct/2021	Added a few more tips
# 1.04 10/Feb/2022	More tips added
# 1.05 11/Mar/2022	Move [tip] call outside of conditional procdef
# 1.06 01/Apr/2022	Added tip help
# 1.07 26/Nov/2022	Added static struct stuff
# 1.08 21/Feb/2023	Minor update
#######################################################################################
# Notes:
# Syntax:  tip | tip -count | tip <number>
#######################################################################################
# When embedding a # character, be sure to dquote rather than wrap in braces
# FIX Broken dquote-embedded */ instances
# Tips are block-wrapped to a fixed width with wordwrap
# [rndseq] is used to ensure we get a sequenced (non repeating) pseudo-random number 
#######################################################################################
# Display all tips...
# loop i 0 [tip -count] {tip $i}
#######################################################################################
option expression off

array unset totd -nocomplain

array setint totd {			# Create an integer-indexed array

# /*
	{Static variables are designed to persist for the life of a script. However they may be unset by using \
the [static -unset] command. (See: help static)}
	"Take care commenting out [if] commands as #if is a macro command!. (See: help macro)"
	{If a program appears not to enter loops or exits unexpectedly check for 'option expression on'. \
You may need to use: 'option expression off'. Eval-based flow-control logic will not work with the former. \
(See: help option expression)}
	{Precalculate values wherever possible, especially outside of loops. (See: help optimisation)}
	{Variables may be 'promoted' to static type using the [static command]. Static variables require use of the [static] command to unset. (See: help static)}
	{Use the command [option debug on] for development level debugging. (See: help option)}
	{The [inspect] command can provide low-level debugging information about a variable. (See: help inspect)}
	{The variable table may be inspected using the [walk] command. (See: array walk)}
	{Arrays can be iterated using the [array foreach] command. (See: array foreach)}
	{Arrays can be inspected using the [array walk] command which displays information about the underlying data. (See: array walk)}
	{Substring position matching can be performed using [instr], [rinstr], [strstr] or [strstrr] (See: help instr)}
	{The [array statistics] command can be used to show information and statistics about an active array. (See: help array statistics)}
	{The command [puts] is used to print output. [printf] can be used to print formatted output. [format] can be used to format strings for [puts]. (See: help puts, help printf)}
	{The [format] command may be used to format strings passed to [puts]. (See: help format)}
	{The [expr] command may be used to evaluate natural expressions. (See: help expr)}
	{Flow control commands such as [if], [for] or [while] can accept either natural expressions or Tcl commands depending on [option expression]}
	{The $this variable is set by [switch], [on] and [goto_block]. (See: help this)}
	{[on] may be used to emulate BASIC's ON GOTO by using an integer value to select a range of script blocks. (See: help on)}
	{BASIC's GOTO command is emulated via [goto_block] with [goto] or [gosub] commands. (See: help goto)}
	{BASIC like subroutines are offered via [sub], [gosub] (See: sub, gosub)}
	{[sub]+[gosub] works either freeform or will call defined blocks within a [goto_block] (See: sub, gosub)}
	{Subroutines may be defined as well as procedures. [sub] defines a procedure with no arguments. Call using [gosub]. (See: help sub)}
	{Fatal Tcl errors may be handled using [catch] or [try..catch]. (See: help catch, help try)}
	{Flow control constructs include [if], [while], [for], [foreach], [switch], [loop], [do], [on], [goto_block], [goto], [gosub]}
	{[do..while] and [do..until] constructs are supported. (See: help do)}
	{[loop] offers a tidier and sometimes more efficient form of [for] loop. (See: help loop)}
	{External DLLs may be called using [calldll] offered by the ticol_calldll plugin. (See: help calldll)}
	{Rudimentary ZIP handling is provided by the ticol_zip.dll plugin. (See: help zip)}
	{Array variables are supported by the [array] command as well as via bracketed variable names such as $a(). (See: help array)}
	{Basic SMTP mail handling is provided by the ticol_mail.dll plugin. (See: help mail)}
	{You can use ticol.exe as an expression evaluator for batch scripts. Use the /EXPR command-line argument (See: help /expr)}
	{Enable [option echo on] when encountering problems and debugging. Command returns are not echoed to the CLI with [option echo off] (See: option)}
	{Command returns are not echoed to the CLI with [option echo off] (See: option)}
	{You'll wish that you had done some of the hard things in life when they were easier to do}
	{Reduce CPU usage in tight loops by using [sleep] or [doevents]. (See: help sleep, help doevents)}
	{Windows system errors can be decoded using [win_error]. (See: help win_error)}
	{Try using the self-optimising [calc] instead of [expr] (See: help calc)}
	{A C-like macro preprocessor is provided. (See: help macro)}
	{Your depth of comprehension may tend to make you lax in worldly ways}
	{You can hurt yourself and others if you run with scissors}
	{The macro preprocessor supports simple macro token subsitution. (See: help macro)}
	{External or separate Tcl files are included using the [source] command. (See: help source)}
	{To relax, imagine you are a loose lily floating down an amber river (See: Black Books)}
	{Any command or procedure may have an alias. This is defined by the [alias] command. Aliases persist until either an aliased lib command is unloaded, or the last command or alias is undefined using [undef]. (See: help alias)}
	{Either [undef] or [rename] may be used to undefine (destroy) procedures and aliases. Rename the command name to "" to undefine. (See: help undef)}
	{If [calc] is used in preference to [expr] any expressions will be automatically optimised unless /NEO is used. (See: help calc)}
	{Ticol is ANSI only and does not support Unicode/MBCS international character sets. Take care when processing files and folders with Unicode/MBCS characters}
	{Ticol does not support regex commands at present. A future version may support simple regex operations}
	{The [item] command is supported either standalone or as a subcommand to many commands as a safe means of accessing objects. (See: help item)}
	{Ticol is an interpreter not a compiler. It will perform much slower than full or bytecode compilers. Performance bottlenecks can be analysed by running in graph mode using ticol.exe /g. (See: help performance)}
	{Ticol Tcl's optional configuration file is called ticol.ini. Ticol expects to find this in the same folder as ticol.exe. (See: help ticol.ini)}
	{Ticol has an optional startup configuration script which is called autoexec.tcl (See: help autoexec.tcl)}
	{Ticol supports CGI webserver use, for non production server use only. (See: help cgi)}
	{Ticol has a built-in single-step debugger. Launch ticol.exe with /DEBUG or /BP. (See: help debugging)}
	{Ticol's debugger supports [halt] and [resume] commands which will enter and leave the debugging state. (See: help halt, help resume)}
	{Performance graph analysis, to determine bottlenecks, may be generated using the /G command-line option. (See: help graph)}
	{Debugging variable watch items may be generated using the [watch] command. (See: help watch)}
	{Try putting lavender in your bath water for a calming effect}
	{Loaded source code and variables may be inspected using the [dump] command. (See: help dump)}
	{Scripts may be loaded into the interpreter using [load]. Loaded or non-loaded scripts may be excuted using [run]. (See: help load, help run)}
	{Binary structs are supported in Ticol via [struct] for use in calling external DLLs via [calldll]. (See: help struct)}
	{Ticol structs may be used to offer a limited form of object orientation. (See: help class example)}
	{User input may be gathered using [input] or [inputbox] which supports password input with masking. (See: help input, help inputbox)}

	{Windows message boxes are supported via the [msgbox] command. (See: help msgbox)}
	{Console text colour may be changed using [textcolor]. (See: help textcolor)}
	{Console text positioning may be set using [gotoxy]. (See: help gotoxy)}
	{ANSI art files or ANSI block text may be displayed using the ticol_ansi plugin. (See: help ansi)}
	{The Ticol Command Line Interface may be used as a Windows shell interface to pass commands through to Windows}
	{The TCL2BAT.EXE or TCL2BAT.BAT batch script may be used to convert a Tcl script into a functional Windows batch script. (See: help tcl2bat)}
	{CGI use will require a specific ticol.ini which is configured for CGI mode. (See: help cgi, help ticol.ini)}
	{[puts] will automatically start a new line unless the '-newline' argument is used}
	{@command or [exec] may be used to execute a Windows command from the command line interpreter (CLI). (See: help @, help exec)}
	{A stitch in time saves nine}
	{The standard release of Ticol Tcl is non-threaded. A purely experimental threaded version exists. (See: help threading)}
	{Cleanup routines may be called by he [at_exit] command. If defined as a proc, this will be called at exit. (See: help at_exit)}
	{Big number support is provided bia the ticol_bignum plugin and the [big] command. (See: help ticol_bignum)}
	{The [chomp] command may be used to efficiently slice up strings. (See: help chomp)}
	{Large strings may be concatenated most efficiently using the [store] command which offers a fast, buffered accumulator. (See: help store)}
	{The Windows clipboard is accessible via the [clipboard] command for basic text access. (See: help clipboard)}
	{Any defined [expr] function may be called directly via the [funct] command. (See: help funct)}
	{CPU information can be obtained via the ticol_cpu plugin. (See: help cpu)}
	{Decimal to Roman conversion is available via the ticol_r2d plugin. (See: help r2d)}
	{DNS lookups, including MX lookups are available via the ticol_dns plugin. (See: help ticol_dns)}
	{DHCP control is accessible via the ticol_dhcp plugin. (See: help dhcp)}
	{Ticol script elevation may be achieved by using the [elevate] command. (See: help elevate)}
	{Moderately secure symmetric encryption is offered via the [encrypt] command. (See: help encrypt)}
	{Script obfuscation is offered via the /C command line argument which can generate obfuscated TCX files using various encryption methods. TCX obfuscated files will be decrypted automatically unless a password is specified}
	{Obfuscated TCX code may be stored, loaded and executed from within a script via the ticol_tcx plugin and [tcx] command. (See: help tcx)}
	{When you have a NULL pointer assignment, why not have a relaxing soak in a warm bath?}
	{Don't eat yellow snow}
	{CRC32 checksums may be generated via the ticol_crc plugin. (See: help crc)}
	{A CRC32 checksums for a file may be generated via the [file_crc] command in the ticol_crc plugin. (See: help crc)}
	{Memory allocation and free is made available for external DLL linkage via the [malloc] and [free] commands. (See: help malloc)}
	{Simple FTP client services are available via the ticol_ftp plugin and [ftp] command. (See: help ftp)}
	{Filenames for open and save may be acquired via the [get_fileopen] and [get_filesave] commands}
	{Some useful CGI routines are made available via the ticol_cgi plugin. (See: help ticol_cgi)}
	{Simple JSON data may be processed via commands in the ticol_json plugin. (See: help json)}
	{HASP licence key monitoring and query is offered via the ticol_hasp plugin. (See: help hasp)}
	{Reprise RLM licence monitoring and query is offered via the ticol_rlm plugin. (See: help rlm)}
	{MD5 support is built-in via the [md5] and [md5_file] commands. Additionally, MD5.EXE is included. (See: help md5)}
	{WAV files may be played using [play_wav] and [stop_wav] commands. (See: help play_wav)}
	{Windows registry access is offered via the ticol_reg plugin. (See: help ticol_reg)}
	{Basic Windows service query commands are provided via the ticol_svc plugin. (See: help ticol_svc)}
	{Random strings may be generated using [randomstr]. (See: help randomstr)}
	{The [scan] command may be used to scan values to one or more Tcl variables as in C/C++. (See: help scan)}
	{A basic pseudo-assembler interpreter is provided via the ticol_asm plugin. (See: help asm)}
	{A brainfuck (BF) intepreter is implemented in the ticol_bf plugin. (See: help bf)}
	{External programs may be run and their results aquired using the [spawn] command. (See: help spawn)}
	{A console shell may be entered via the [shell] command. (See: help shell)}
	{External programs may be executed via the [exec] command (See: help exec, help spawn)}
	{Rudimentary variant access is provided via the ticol_variant plugin. (See: help variant)}

	{DBase III / IV files may be manipulated via the ticol_xbase plugin. (See: help xbase)}
	{A safe default for array access is to use [array item] with the -nocomplain argument. If the array or element does not exist this can return a default value or the internal default of {} of not specified. (See: help array item)}
	{[concat] may be used to efficiently concatenate multiple strings. See also: [store] for efficient buffered storage. (See: help concat)}
	{[slice] may be used to 'slice' well-formed multi-line strings into blocks and output as Tcl lists. (See: help slice)}
	{To exit the Command Line Interpreter (CLI), use exit or CTRL+BREAK. (See: help exit)}
	{Integer indexed arrays may be generated from well-formed Tcl lists using [array setint]. (See: help array setint)}
	{Constants may be defined using the [const] or [enum] commands (See: help const, help enum)}
	{Enumerated constants are supported via the [enum] command. (See: help enum, help const)}
	{Multi-dimensional arrays are supported as per standard Tcl using emulated subscript separators. (See: help arrays)}
	{Ticol arrays use hashtables for fast access and work efficiently with very large arrays of a million or more elements. (See: help arrays)}
	{Dictionaries use hashtables for top level storage and unless deeply-nested will be efficient even with large dictionaries}
	{It is not permissible to undefine a proc or a proc's parent while active. Test using [is_active]. (See: help undef, help is_active)}
	{If an attempt is made to undefine a proc while an alias is active an error will be raised and the proc will remain active until the alias is undefined. (See: help undef)}
	{Code syntax highlighting is provided via the ticol_highlight plugin. (See: help ticol_highlight)}
	{The [static] command is available for use within procedures. This maps to a variable in the global namespace. (See: help static)}
	{Namespaces other than the global (::) namespace are emulated in Ticol. (See: help namespace)}
	{Before defining a new [proc] you can use the [defined] test to decide whether to define or undefine an existing instance. (See: help defined)}
	{The "SEP field" has more of a relevance in medical diagnosis than perhaps is realised, particularly when it \
comes to unusual/non-canonical phenomena and where it may be easier and more convenient for a practitioner \
to surround a problem with a modest SEP field in order to toss it into the long grass of psychiatric \
classification. As Adams pointed out, SEP fields have fairly low energy requirements and have inherent \
plausibility}
	{Ticol Tcl default integer size is 64-bit. You can cast to other widths using cast commands. The ticol_bignum plugin can be used to handle larger integers. (See: help data type ranges)}
	{The [expr] command will reorganise expressions using BEDMAS but brackets are still recommended. (See: help bedmas)}
	{The CLI prompt is customisable via the ticol.ini configuration file. (See: help ticol.ini)}
	{Scientific format numbers are handled via [atof], [fota] and [sci], [dec] in the ticol_sci plugin. (See: help sci, help atof)}
	{Windows event logging is offered via the ticol_event plugin. (See: help ticol_event)}
	{Windows drive-letter mapping is provided via the ticol_mapdrive plugin. (See: help ticol_mapdrive)}
	{SQLite SQL is provided via the ticol_sql plugin. (See: help ticol_sql)}
	{Console-compatible 2-colour bitmap handling is provided via the ticol_bmp plugin. (See: help ticol_bmp)}
	{C/C++ like arrays may be manipulated via the ticol_carray plugin. (See: help ticol_carray)}
	{[struct] can be used to create C/C++ binary-compatible arrays which can be passed to external DLLs. (See: emulating C arrays)}
	{The ticol_carray plugin can offer non-associative 1, 2 and 3 dimensional arrays. (See: help ticol_carray)}
	{[array walk] can be used for debug inspection of an array's hashtable contents. (See: help array walk)}
	{The [dump] command allows variable contents to be inspected. (See: help dump)}
	{[struct dump] may be used to generate a detailed debugging analysis of a struct's data. (See: help struct)}
	{[time] may be used to execute a block of code a given number of times and produce an analysis of execution time on completion. (See: help time)}
	{Kernighan and Richie (K&R) bracing style must be used for Tcl. It is not optional. Allman/BSD style bracing must not be used. (See: help braces)}
	{The Command Line Interpreter (CLI) will ususally show the command count. This may be reset or inspected using [cmdcount]. (See: help cmdcount)}
	{Constant arrays are supported. Append the argument '-const' after [array set]. e.g. array set a {foo bar} -const. (See: help arrays)}
	{Double $$ dereferencing and multi-level dereferencing is offered as an enhancement over standard Tcl dereferencing. (See:  help double dereference)}
	{The standard Tcl '{*}' (wart) expansion operator is supported. This will expand a list/string into a command array. (See: help expand)}
	{Use the up and down cursor keys to search the Ticol CLI command history (See: help cli)}

	{A [sub] may return an optional value but cannot take arguments. A [sub] may access global variables. [sub] may still use [upvar] to reference variables in parent scope. (See: help sub)}
	{The PERL 'spaceship' operator, '<=>' is implemented. This retuns a 'signed boolean' (-1, 0, 1) when comparing strings (See: help <=>)}
	{The 'K-combinator' is implemented as [K] for use with Lambda style operations. (See: help K)}
	{The Tcl command-separator is the semi-colon as with C/C++. Multiple commands may thus be placed on a single line. (See: help semi colon)}
	{Assignmnet in Tcl is performed using the [set] command, for example: set a 10, and an enhanced [let] command may also be used. (See: help set, help let)}
	{The [let] command can be used for 'smart' assignment either with or without evaluation of the Rvalue. (See: help let)}
	{The [let] command can assign to multiple objects at once. (See: help let)}
	{Global variables do not 'inherit' into local procedure scope. They may be accessed using the global scope prefix '::' or via the [upvar] command. (See: help upvar)}
	{[cmp] is recommended for comparing strings by specified length. (See: help cmp)}
	{[setat] is an efficient command for replacing sections of a string with other strings: (See: help setat)}
	{[replacechar] is an efficient command for replacing characters within a string: (See: help setat)}
	{String comparison commands [eq], [ne], [gt], [ge], [lt], [le], [eqn], [nen], [eqi], [nei], [lti], [gti], [gei] and [lei] are available. (See: help comparison commands)}
	{Be sure to brace the variable list in [foreach]. e.g. as 'x' and 'y' in [foreach {x y} $list {...} ]}
	{Always properly brace argument groups in the [if], [for] and similar commands. (See: help braces)}
	{It is permissible (but not recommended) for a procedure to have no name but it must be called using [call]; as in: proc {} {} {puts hi};call {}. (See: help call)}
	{A procedure may have a valid name comprised of one or more whitespace characters, but whitespace in procedure and variable names is not recommended. (See: help proc)}
	{Lambda like operations may be performed: (See: help lambda)}
	{Sometimes you just look at yourself in the mirror after the 400th bottle of corner-shop wine and think... This is fantastic}
	{PERL-style multiple-dereferencing is supported using the dollar sign  [set [set [set a]]]  is the same as $$$a. (See:  help double dereference)}
	{The [assert] command can be useful during development and debugging of scripts: (See: help assert)}
	"Ticol uses the '@' symbol for [upvar] and [uplevel] level addressing instead of the '#' comment character. The # character is reserved for comments and macro commands"

	{Braced variables are dereferenced recursively with the innermost braces resolved first. This allows complex multi-level dereferencing\
The opening brace must follow immediately after the dollar $ sign\
Spaces are within curly-braces are treated as part of the variable name and a variable name may be one or more spaces}
	{argv() may be safely tested using [item] which can return a NULL (or other) default if not set. (See: help item)}
	{ticol.ini can be used to configure a number of default options, including CGI configuration. (See: help ticol.ini)}
	{Ticol Tcl variable and procedure names are limited to A-Z a-z 0-9 _ . : () & <tab> and <space>. (See: help variable names)}
	{String delimiters need careful attention in Tcl. Tcl is not C++. Strings don't always need to be delmited. (See: help quoting hell)}
	{Minimal escaping style should be embraced for Tcl. This means using backslash escapes as rarely as possible. (See: help minimal escaping)}
	{Ticol features optional restricted execution which may bind to workstation, username, IP, password etc. using TCX files. (See: help tcx)}
	{Ticol offers VB compatible date and time routines. (See: help date)}
	{The environment table is available via the const $env() array. (See: help env)}
	{Standard C file handling routines are available via [file], [read], [write] etc. (See: help file)}
	{[let] can be used for assignment as an alternative to [set]. (See: help let)}
	{Unlimited multi-line strings can be created using the backslash end of line continuation character. (See: help line continuation)}
	{You are never in more danger when you are right and the government is wrong}
	{[static] can be used to create static variables for procedures. This creates persistent variables in the global namespace. (See: help static)}
	{Static variables are created in the root namespace and are protected from deletion for the lifetime of the script}
	{Static variables are created in the root namespace and have a name comprised of the proc name plus the variable name (See: help static)}
	{Scripts may self-elevate Windows security using the [elevate] command. (See: help elevate)}
	{The $env() array is case insensitive with respect to subscript values. Case insensitive arrays may be created using [array set] with '-nocase' argument. (See: help array set)}
	{Case insensitive arrays may be created using [array set] with '-nocase' argument. (See: help array set)}
	{The find command may be used to search and topics in the manual file. (See: help find)}
	{The Ticol Command Line Interpreter (CLI) may handle backslash escapes differently depending on the [option] seting. Certain differences are to allow the CLI to function as a Windows shell. (See: help cli)}
	{Ticol Tcl does not support classes and objects directly but structs could be used to emulate this. (See: help class example)}
	{Ticol works well with Apache CGI and configuration examples are included with the Ticol distribution. Apache security should be properly configured. (See: help cgi)}
	{Backslash character handling differs in scripts and the Command Line Intepreter. (See: help backslash)}

	{Experience the sheer joy of having a pickle with your sandwich}
	{The [uplevel] command allows a command script to be executed in a scope other than the current innermost scope on the stack. (See: help uplevel)}
	{The [upvar] command creates a link to variable in a different stack frame, either by relative or absolute reference. (See: help upvar)}
	{The [try] (...catch) command lets you trap and process errors and exceptions. (See: help try)}
	{Ticol does not yet implement the [regexp] command. This matches a regular expression against a string.}
	{A demonstration Dbase viewer script is included as dbview.tcl}
	{Caution: A limit of Ticol is that it handles the ANSi character set only. Non-ANSI (Unicode/MBCS) characters are not handled. (See: help unicode)}
	{INI file manipulation is available via the ticol_ini plugin. (See: help ini)}
	{Filename selection for both file open and save can be achieved using the Windows dialogs offered by [get_openfile] and [get_savefile] (ANSI filenames only). (See: help get_openfile, help get_savefile)}
	{Folder selection can be achieved using the Windows dialog offered by [get_openfile] and [get_savefile]. (See: help get_openfile, help get_savefile)}
	{External VBScript code may be executed either by using the ticol_cscript plugin or via [spawn] calling cscript.exe: (See: ticol_cscript)}
	{A __stdcall DLL version of the Ticol Tcl interpreter is available. This can be called from Visual BASIC, C++ or via the ticol_stub.exe test program. (See: ticol.dll)}

	{The TCL2C.EXE utility can be used to generate a C++ variable block from Tcl code in order to embed into a C++ program (See: help tcx)}
	{Obfuscated TCX scripts can be execution-locked to a specific PC or by password, MAC address etc. (See: help tcx)}
	{You can create a loadable obfuscated TCX block by 'compiling' a TCL file adding the /64 argument to /C (See: help tcx)}
	{Tcl looks a little like C/C++ but the language structure is quite different. Don't treat Tcl as if it were C++}
	{Always wrap command arguments and proc definitions in braces. Avoid the temptation to omit braces within say, [if] ladders}
	{Happiness is when what you say and what you do are in harmony}
	{Braces serve a specific function in Tcl. They delay (block) evaluation of the braced contents}
	{When you are in a tight spot debugging a script, try to imagine yourself marooned on a desert island}
	{If a script halts or exits for no apparent reason use [option echo on], make full use of [catch] and [try catch], use the debugger to single-step by adding /BP to ticol.exe command line or using [option breakpoint on] and, if necessary, a [halt] statement before the suspected error. You can also use the /DEBUG argument for ticol.exe (See: help troubleshooting)}
	{Some Tcl 'long form' math commands such as [not], [and], [xor] etc. are not implemented as words. Use symbols such as [!], [&] (See: help math)}
	{Function (command) call tracing can be used to help debug program flow using the trace command. (See: help trace)}
	{Self-elevating scripts can be constructed by using [is_elevated] and [elevate] (See: help elevate)}
	{Use file-level modularity and [source] to break up scripts and avoid large, monolithic script files (See: help source)}
	{You can use ticol.exe as an expression evaluator for batch scripts}
	{Although [expr] will attempt to apply BEDMAS rules, you are recommended to make full use of brackets in expressions}
	{You can initialise using data from another file by using [source scriptname] (See: help source)}
	{Return from a child script to the parent using [return -code exit] (See: help return)}
	{You can make a new array constant by appending -const to the array declaration (See: help array)}
	{You can check at runtime if the current active script was encrypted using [info encrypted] which will return a boolean}
	{It is possible to write limited scripts entirely using the Macro PreProcessor (MPP) since these can also call Tcl commands}
	{Braced variables are dereferenced recursively with the innermost braces resolved first. This allows complex multi-level dereferencing}

	{Windows NT family version as far back as NT4/SP6a and Windows 2000 are supported as far as possible. Ticol is also ReactOS compatible}
	{True floating-point operations are not yet supported by the Big Math plugin (See: help bigmath)}
	{Braces do not group within array subscripts of string literals containing whitespace (See: help array)}
	{If things don't work as expected always first check your [option expression] setting (See: help option)}
	{Introspection is provided via a number of mechanisms such as [trace], [option debug]+[halt], [watch], [walk], /g graph}
	{Use [rndseq] to get a pseudo-random number which is guaranteed not to repeat (See: help rndseq)}
	{Procedures can be defined within procedures but remember to clean up and [undef] a nested procedure when you exit (See: help undef)}
	{Command (or proc) aliases may block the undefinition of a procedure. This may be problematic for procedures defined locally within another proc (See: help alias)}
	{Very large, sparse integer indexed arrays may be quickly indexed by using a secondary array which can be sorted and output (See: array sort)}
	{You can load a program into the CLI using:   load <scriptname>   The loaded program can be *.tcl or *.tcx (See: help load)}
	{You can inspect the loaded/macro-processed script from the CLI using:   dump  (See: help dump)}
	{The [run] command will allow a script to be loaded and executed with optional command-line arguments (See: help run)}
	{Self-elevating scripts can be constructed by using [is_elevated] and [elevate] (See: help elevate)}
	{All generalizations are false, including this one (Mark Twain)}
	{You can 'compile' a TCL script into a self-executing batch (BAT) file using the tcl2bat.exe utility. (See: help tcl2bat)}
	{Use file-level modularity to break up scripts and avoid large, monolithic script files. (See: help source)}
	{Break big scripts into smaller files and call using [source scriptname]. Be aware that each call to [source scriptname] will evoke the macro preprocessor (MPP) which has a finite time cost (See: help source)}
	{Topic help is available from the Ticol command-line using:   help <topic>   Topics with spaces may be wrapped in double quotes but it isn't essential}
	{If you're not sure of the help topic use find, e.g.:  find string (See: help find)}
	{Padded or aligned strings can be created using [setl], [setr] and [setc] which will align one string either left, right or centrally within another (See: help setl, help setr, help setc)}
	{An enhancement over [puts] and [format] is [printf] which works similarly to the C/C++ printf() (See: help format, help printf)}
	{Although [expr] will attempt to apply BEDMAS rules, you are recommended to make full use of brackets in expressions (See: help expr)}
	{You can initialise using data from another file by using [source scriptname] (See: help source)}
	{You can profile script performance and spot bottlenecks using the /G command-line option}
	{Portions of strings may be replaced using either [mid] or [setat] (See: help mid, help setat)}
	{[is_mod] offers a simpler and quicker way to take actions depending on a mod result (See: help is_mod)}
	{Return from a child script to the parent using [return -code exit] (See: help return)}
	{You can make a new array constant by appending -const to the array declaration (See: help array)}
	{Differences between arbitrary dates can be calculated by subtracting the result of [date] (See: help date)}

	{Real numbers may be truncated to a given level of precision using [trunc] (See: help trunc)}

	{2D and 3D arrays can be manipulated using the [carray] plugin libarary - ticol_carray.dll (See: help carray)}
	{the effective precision of a real number may be returned using [precision] (See: help precision)}
	{IP addresses can be matched using wildcards via the ticol_misc plugin and its [ip_match] command (See: help ip_match)}
	{An IP address may be matched to a CIDR mask using [cidr_match] in the ticol_misc plugin (See: help cidr_match)}
	{Symmatric Rot13 encoding is supported via [rot_13] in the ticol_masc plugin (See: help rot_13)}
	{Nested macro if, else, endif statements are not currently supported (See: help mpp)}
	{The /ECHO command line argument may be use to echo the Macro-Preprocessed source code to the console. (See: help echo)}
	{Tcl does not lend well to C-style binary arrays of struct. Arrays of struct can be emulated by storing the struct instance name and dereferencing via $array(subscript).structname (See: help arrays of struct)}
	{To avoid dereferencing a variable containing a dot and literal, isolate the variable using curly braces (See: arrays of struct)}
	{Array elements can be exchanged using [array swap] (See: help array swap)}
	{A cleanup command can be called when any error occurs by defining an [on_error] sub or proc (See: help on_error)}
	{Although Ticol performs 64 bit internally, many commands accept and return only 32 bit arguments}
	{Lists can be handled using push or pop operations via [lpush] and [lpop] (See: help lpush, help lpop)}
	{Evaluation for square brackets is performed in the reverse order to that which might be expected. For example, the statement:\	
puts [call [return]] \
Will be evaluated in the order of innermost-to-outermost brackets (See: help square brackets)}

	###################################################################################################
	# FIX * / within dquotes... Oct 2021
	# Strings with special characters which require MPP protection by being wrapped in dquotes
#*/

# OCT 2021 FIX long comment handling in dquotes. Fix terminator clause handling or DOC (see source)
	"C/C++ style long comments /* .. */ are supported. (See: help comment)"
#	"Long comment /* .. */ tags have no effect within double-quoted strings as long as a long comment is not active. (See: help comments)"		# Ensure this is dquoted not braced
# /*
	"Source code line numbers can be entered into a script using the #__LINE__ macro (See: help macro)"
	"Line numbers may be entered into scripts using the #__LINE__ macro. (See: help macro constants)"
	"Tcl # comments and C-like long comments may be mixed together. (See: help comments)"
	"The #exec macro command may be used to execute a Tcl command during macro preprocessing, which is before the script is executed. (See: help exec macro)"
	# When running from the CLI, backslashes may not be interpreted. This depends on the [option] setting
	"Multi-line strings may be defined using the backslash \\ line-continuation character place at the end of a line"	# Comment
# */		
	"Ticol uses the @ sign rather than # sign in [upvar] and [uplevel] to specify an absolute level (See: help upvar)"
}

#######################################################################################
halt
# option break on
#/*
if {! [defined tip]} {
	proc tip {args} {
		# puts "[is_set args]"
		# puts "args: $args '[list $args]'"

		try {	
				if {[in $args {-count} -nocase]} {					# tip -count
					# puts "[array size ::totd] tip(s)"				# Alternate method
					return [array size ::totd]
				} elseif {any [eqi $args "help"] [eqi $args "-help"] [eqi $args "/?"]} 	{
					textcolor white
					puts "Syntax: tip ?n? ?-count?"
					textcolor
					newline
					return
				} elseif {[is_numeric $args]} 	{
					if [array exists ::totd] {
						set screen_width [max 40 [- [console width] 5]]
						set r [min $args [- [array size ::totd] 1]]
						set r [max $args 0]
						textcolor white
						puts [wrap "Tip: [array item ::totd $r "Use help to get help"]" $screen_width -wordwrap]
						textcolor
						newline
					} else {
						puts "Array totd() does not exist. It may have been cleared"
						puts "Reload script \"[info script]\""
					}
				} else 	{											# tip
					# option escape push on
					set screen_width [max 40 [- [console width] 5]]
					if [array exists ::totd] {
						textcolor white
						puts [wrap "Tip: [array item ::totd [rndseq 0 [- [array size ::totd] 1]] "Use help to get help"]" $screen_width -wordwrap]
						textcolor
						newline
					} else {
						puts "Array totd() does not exist. It may have been cleared"
						puts "Reload script \"[info script]\""
					}
				}
		} catch {
			puts "Error in script totd.tcl"
			die
		}
	}
}

tip 					# Display Tip of the Day
textcolor yellow
puts "Use the command 'tip' for another tip"
textcolor
newline

# */

# EOF #